
# q-expansion of newform 30.4.a.b, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 30
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[2], [3], [-5], [-4], [-48], [2], [-114], [140], [72], [210], [272], [-334], [-198], [-268], [216], [-78], [240], [302], [596], [-768], [-478], [-640], [-348], [210], [-1534], [1722], [1052], [-564], [-610], [1302], [-124], [192], [-2514], [1340], [1410], [-2128], [3026], [2612], [-24], [1962], [-120], [902], [-168], [-1318], [-4014], [2000], [-3868], [-3148], [2556], [-610], [-2058], [4920], [-1438], [792], [2166], [3192], [5490], [-6328], [-574], [4242], [-628], [-558], [-6964], [2832], [8642], [-2214], [10772], [-1654], [2196], [8270], [10302], [-2280], [-8764], [-1318], [1100], [-3528], [-9630], [-3094], [-1638], [-13750], [-12480], [7262], [9792], [1802], [-2320], [11172], [6810], [17066], [-18918], [1052], [11076], [-9000], [-8764], [5592], [4700], [-11808], [1170], [-16638], [15692], [-22018], [-4564], [-7734], [-20148], [-24030], [2372], [8546], [-15444], [18342], [24600], [-8998], [4076], [-4078], [10086], [8780], [2792], [7602], [24212], [9456], [-9558], [-29280], [-29098], [-11638], [3426], [-20148], [-29428], [16242], [2030], [6960], [18596], [21242], [-340], [-21888], [17792], [37346], [-11358], [-34270], [-13278], [-11164], [-5094], [-8790], [5852], [-29478], [39332], [6756], [3950], [12360], [-35998], [-21594], [9260], [31632], [-39694], [1242], [-2668], [-4344], [4436], [22752], [-27160], [-33030], [-29974], [13962], [35196], [-28338], [-17524], [-26808], [-10914], [22272], [14072], [4826]]
